@@ -1,5 +1,5 @@
 
-# $Id: AP.pm,v 1.3 2003-12-13 16:13:47-05 kingpin Exp kingpin $
+# $Id: AP.pm,v 2.103 2004/05/23 03:15:47 Daddy Exp $
 
 =head1 NAME
 
@@ -29,6 +29,9 @@ To make new back-ends, see L<WWW::Search>.
 
 =head1 NOTES
 
+This backend (and all its subclasses) only searches news stories from
+the last 14 days.
+
 =head1 BUGS
 
 Please tell the author if you find any!
@@ -65,27 +68,18 @@ use WWW::Search::WashPost;
 
 @ISA = qw( WWW::Search::WashPost );
 
-$VERSION = '2.01';
+$VERSION = do { my @r = (q$Revision: 2.103 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 $MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
 
 # private
 sub native_setup_search
   {
   my ($self, $sQuery, $rhOptions) = @_;
-  $self->{_options} = {
-                       'search_url' => 'http://sitesearch.washingtonpost.com/cgi-bin/search99.pl',
-                       'searchsection' => 'news',
-                       'searchtext' => $sQuery,
-                       'searchdatabase' => 'ap',
-                      };
+  $rhOptions->{'source'} = 'APOnline';
+  # All further work is done by our superclass, WWW::Search::WashPost:
   $self->SUPER::native_setup_search($sQuery, $rhOptions);
   } # native_setup_search
 
-
 1;
 
 __END__
-
-http://www.washingtonpost.com/cgi-bin/search99.pl?searchdatabase=serf&serf_ap=on&ap=on&description=microsoft+eu+concessions&headline=&_u.16=14
-
-http://sitesearch.washingtonpost.com/cgi-bin/search99.pl?searchdatabase=serf&serf_ap=on&ap=on&description=%22World+Cup%22&headline=&_u.16=14
